@@ -23,7 +23,6 @@ class SaintDataset(Dataset):
         self.X_categorical = self._define_tensor_features(data, cat_cols, torch.int64)
         self.X_continuos = self._define_tensor_features(data, con_cols, torch.float32, scaler)
         self.y = self._define_tensor_target(data, target, target_categorical)
-        self.prediction_column: Tensor = rearrange(torch.zeros(data.shape[0], dtype=torch.int64), 'n -> n 1')
 
     def __len__(self):
         return len(self.y)
@@ -70,5 +69,5 @@ class SaintDataset(Dataset):
 
         :param idx: numeric index of the data that we want to process
         """
-        return torch.cat([self.X_categorical[idx], self.prediction_column[idx]]), self.X_continuos[idx],\
+        return self.X_categorical[idx], self.X_continuos[idx],\
             self.y[idx]
