@@ -2,6 +2,7 @@ from typing import List, Tuple, Optional
 
 import torch
 import torch.nn.functional as f
+from einops import rearrange
 from pytorch_lightning.core import LightningModule
 from torch import nn, Tensor
 import numpy as np
@@ -20,6 +21,7 @@ class SAINT(LightningModule):
     :param dim_target: if categorical represent number of classes of the target otherwise is 1
     :param config: configuration of the model
     :param pretraining: boolean flag, if True it will be executed pretraining task
+    :param mc_dropout: boolean flag, if true mc_dropout will be applied during prediction
     """
     def __init__(
             self,
@@ -28,9 +30,11 @@ class SAINT(LightningModule):
             dim_target: int,
             config: SaintConfig,
             pretraining: bool = False,
+            mc_dropout: bool = False
     ):
         super().__init__()
         assert all(map(lambda n: n > 0, categories)), 'number of each category must be positive'
+        self.mc_dropout = mc_dropout
         self.config = config
         self.pretraining = pretraining
         self.dim_target = dim_target
@@ -52,7 +56,10 @@ class SAINT(LightningModule):
         self._define_transformer()
         self._define_mlp(categories)
         self._define_projection_head()
-        self.mlpfory = SimpleMLP(self.config.network.embedding_size, 1000, self.dim_target)
+        self.mlpfory = SimpleMLP(self.config.network.embedding_size * self.num_columns,
+                                 self.config.network.internal_dimension_output_layer, self.dim_target,
+                                 dropout=self.config.network.ff_dropout,
+                                 output_layer=nn.Softmax() if self.dim_target > 1 else None)
 
     def _define_transformer(self) -> None:
         """Instantiate the type of Transformed that will be used in SAINT"""
@@ -68,7 +75,9 @@ class SAINT(LightningModule):
     def _define_embedding_layers(self) -> None:
         """Instatiate embedding layers"""
         # embed continuos variables using one different MLP for each column
-        self.embedding_continuos = nn.ModuleList([SimpleMLP(1, 100, self.config.network.embedding_size)
+        self.embedding_continuos = nn.ModuleList([SimpleMLP(1, self.config.network.internal_dimension_embed_continuous,
+                                                            self.config.network.embedding_size,
+                                                            dropout=self.config.network.ff_dropout)
                                                   for _ in range(self.num_continuous)])
         # embedding layer categorical columns
         self.embedding_categorical = nn.Embedding(self.num_unique_categories, self.config.network.embedding_size)
@@ -83,10 +92,12 @@ class SAINT(LightningModule):
         """Define projection heads for contrastive learning"""
         self.pt_mlp = SimpleMLP(self.config.network.embedding_size * self.num_columns,
                                 6 * self.config.network.embedding_size * self.num_columns // 5,
-                                self.config.network.embedding_size * self.num_columns // 2)
+                                self.config.network.embedding_size * self.num_columns // 2,
+                                dropout=self.config.network.ff_dropout)
         self.pt_mlp2 = SimpleMLP(self.config.network.embedding_size * self.num_columns,
                                  6 * self.config.network.embedding_size * self.num_columns // 5,
-                                 self.config.network.embedding_size * self.num_columns // 2)
+                                 self.config.network.embedding_size * self.num_columns // 2,
+                                 dropout=self.config.network.ff_dropout)
 
     def _embed_data(self, x_categ: Tensor, x_cont: Tensor) -> Tuple[Tensor, Tensor]:
         """Converts categorical and continuous values in embeddings
@@ -226,8 +237,8 @@ class SAINT(LightningModule):
         reps = self.transformer(x_categ_enc, x_cont_enc)
         # select only the representations corresponding to y and apply mlp on it
         # in the next step to get the predictions.
-        y_reps = reps[:, self.num_categories - 1, :]
-        y_outs = self.mlpfory(y_reps)
+        reps = rearrange(reps, 'b h n -> b (h n)')
+        y_outs = self.mlpfory(reps)
         return y_outs
 
     def pretraining_step(self, x_categ: Tensor, x_cont: Tensor) -> Tensor:
@@ -257,7 +268,8 @@ class SAINT(LightningModule):
         else:
             y_pred = self(x_categ, x_cont)
             if self.dim_target > 1:
-                return self._classification_loss(y_pred, target)
+                loss = self._classification_loss(y_pred, target)
+                return loss
             else:
                 return self._regression_loss(y_pred, target)
 
@@ -278,10 +290,13 @@ class SAINT(LightningModule):
 
     def predict_step(self, batch: Tuple[Tensor, Tensor, Tensor], batch_idx: int,
                      dataloader_idx: Optional[int] = None) -> Tensor:
+        if self.mc_dropout:
+            for m in self.modules():
+                if m.__class__.__name__.startswith('Dropout'):
+                    m.train()
         x_categ, x_cont, _ = batch
         y_pred = self(x_categ, x_cont)
-        # return the class that has the greatest probability
-        return y_pred.argmax(1) if self.dim_target > 1 else y_pred
+        return y_pred
 
     @staticmethod
     def _classification_loss(y_pred: Tensor, target: Tensor) -> Tensor:
@@ -290,6 +305,7 @@ class SAINT(LightningModule):
         :param y_pred: Values predicted
         :param target: Values to predict
         """
+        nn.CrossEntropyLoss()
         return f.cross_entropy(y_pred, target)
 
     @staticmethod
